;################### Header File for Customer Firmware ####################
;---- File    	   : AS6031_AS6040_User_FW.h
;---- Date	 	   : 2020/06/04
;---- Company      : ScioSense B.V.
;---- Product      : AS6031, AS6040
;----
;---- Description  : Declarations of variables, bit numbers and constants
;
;#############      Naming conventions    #######################
; RAM_.... general RAM addresses  0x00 - 0xAF
; RAM_R_... RAM Addresses and constants used in ROM Routines
; RAM_VXX_... Temporary RAM variable addresses, same number XX indicates same address
; FDB_... Firmware data buffer, part of the RAM temporarily used by the front-end (0x80 - 0x9B)

; FWD_... Firmware data, user configuration input 0x100 - 0x17F; 0x16C - 0x17F used for input of config registers
; FWD_R_... Firmware data registers used by ROM routines

; BNR_... Bit number within a data word

;#############      Memory organisation      #####################
;
; Place here your user constants
CONST RAM_R_FLOW_LPH				0x02			;---- Calculated flow volume (l/h) with fd 16 without filtering

CONST RAM_R_TOF_DIFF				0x07			;---- TOF Difference (not offset corrected)
CONST RAM_R_TOF_HIT_NO				0x08			;---- TOF Hit Number
CONST RAM_R_FLOW_SCALE_FACT			0x0A			;---- Factor to scale the flow for the pulse interface

CONST RAM_R_V1F_SHIFT				0x1F			;---- Temporary variable for rapid shifting, used in ROM_FORMAT_64_TO_32BIT

CONST RAM_R_TDC_PERIOD				0x5E			;---- TDC Period = (250 or 125 * 10^-9 * 2^32); This variable considers HSC calibration
CONST RAM_R_TDC_CLK					0x5F			;---- Global variable : TDC clock = (4MHz = 0x003D0900 | 8MHz = 0x007A1200); This variable considers HSC calibration

CONST RAM_R_V9E_MEAS_RATE_INV		0x9E			;---- Inverse of the calculated measurement rate
CONST RAM_R_VA5_TOF_TO_FLT_U		0xA5			;---- Normalised TOF Up value before filtering
CONST RAM_R_VA6_TOF_TO_FLT_D		0xA6			;---- Normalised TOF Down value before filtering

;-----------------------------FIRMWARE DATA MEMORY : 0x102 - 0x16B--------------------------------------
CONST FWD_SIMPLE_SCALE				0x102			;002---- Scaling for Diff_ToF to flow rate l/h, fd 16 [l/h*1/ns]
CONST FWD_ZERO_OFFSET				0x103			;003---- Zero flow offset at calibration from Diff_ToF, is used for offset correction, TDC raw value format
CONST FWD_MAX_TOF_DIFF				0x104			;004---- Maximum allowed TOF_DIFF value for further processing
CONST FWD_NEG_TOF_DIFF_LIMIT		0x105			;005---- Limit for negative TOF_DIFF or corresponding negative flow rate for further processing

CONST FWD_R_PULSE_PER_LITER			0x15B			;091---- Firmware data memory with the number of pulses per litres for the Pulse Interface
CONST FWD_R_PULSE_MAX_FLOW			0x15C			;092---- Firmware data memory with the maximum flow in litres

CONST FWD_USM_RLS_DLY_INIT			0x167			;103---- USM_RLS_DLY Init value
